/*  Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
namespace MpcSharp
{
    using System;
    using MPD;

    public class Connection
    {
        public  MPDClient  client;
        public  StatusInfo statusInfo = 
            new StatusInfo(Functions.GetHashtableFromBuffer(""));
        public  StatusInfo prevStatusInfo =
            new StatusInfo(Functions.GetHashtableFromBuffer(""));
        public  StatsInfo  statsInfo =
            new StatsInfo(Functions.GetHashtableFromBuffer(""));
        public  StatsInfo  prevStatsInfo =
            new StatsInfo(Functions.GetHashtableFromBuffer(""));
        public  MusicInfo  musicInfo;
        private CommandCallback callback;
        public delegate void UpdateDelegate();
        UpdateDelegate updateDelegates;

        // Constructor
        public Connection()
        {
            callback = new CommandCallback(ResultCallback);
        }

        // Create the connection to mpd. Return true on success.
        public bool Connect()
        {
            try
            {
                client = new MPDClient(Settings.Host, Settings.Port);
            }
            catch (MPDException)
            {
                return false;
            }
            catch (System.Net.Sockets.SocketException)
            {
                return false;
            }

            // Send the password if specified
            if (Settings.Password != "")
            {
                try
                {
                    client.Password(Settings.Password);
                }
                catch (MPDException)
                {
                    return false;
                }
            }
            return true;
        }

        // Process the return info from mpd
        void ResultCallback(object result)
        {
            if (result is StatusInfo)
            {
                prevStatusInfo = statusInfo;
                statusInfo = (StatusInfo) result;
            }
            else if (result is StatsInfo)
            {
                prevStatsInfo = statsInfo;
                statsInfo = (StatsInfo) result;
            }
            else if (result is MusicInfo)
                musicInfo = result as MusicInfo;
        }

        public bool Refresh()
        {
            // Update status info
            try
            {
                client.Status(callback);
                while(client.ReadResponse())
                {}

                // Update stats info
                client.Statistics(callback);
                while(client.ReadResponse())
                {}

                // Update info about currently playing song
                client.CurrentSong(callback);
                while(client.ReadResponse())
                {}
            }
            catch (MPDException e)
            {
                client.CleanCache();
                // Something has gone wrong, set client to null to force a new
                // connection to mpd.
                client = null;
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
                return false;
            }
            catch (Exception e)
            {
                client = null;
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
                return false;
            }
            return true;
        }

        // Register a new update method
        public void RegisterUpdateDelegate(UpdateDelegate d)
        {
            updateDelegates += d;
        }

        public void UnregisterUpdateDelegate(UpdateDelegate d)
        {
            updateDelegates -= d;
        }


        // Run update delegates
        public void RunUpdateDelegates()
        {
            if (updateDelegates != null)
            {
                updateDelegates();
            }
        }
    }
}
