/*  Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using Gtk;
using MPD;

namespace MpcSharp
{
    //class Find : VBox, IUpdatable
    class Find : VBox
    {
        Connection conn;
        Glade.XML gxml;
        Glade.XML gxml_menu;
        ListStore tree;
        CommandCallback entryCb;
        ComboBox combobox;

        [Glade.Widget] TreeView view;
        [Glade.Widget] Entry entry;
        [Glade.Widget] HBox hbox;

        public Find(ref Connection conn) : base()
        {
            this.conn = conn;
            gxml = Glade.XML.FromAssembly("find.glade", "hbox", null); 
            gxml.Autoconnect(this);
            PackStart(gxml.GetWidget("hbox"), false, false, 0);

            gxml = Glade.XML.FromAssembly("find.glade", "scrolled_win", null); 
            gxml.Autoconnect(this);
            PackStart(gxml.GetWidget("scrolled_win"));


            gxml_menu = Glade.XML.FromAssembly("find.glade", "find_popup", null); 
            gxml_menu.Autoconnect(this);

            entryCb = new CommandCallback(SearchResultsCb);

            tree = new ListStore(typeof(string));
            view.Model = tree;
            view.AppendColumn("Results", new CellRendererText(), "text", 0);
            view.Columns[0].Sizing = TreeViewColumnSizing.Autosize;
            view.Selection.Mode = SelectionMode.Multiple;

            // Fill combo box with metadata types
            combobox = ComboBox.NewText();
            foreach (string item in SearchType.GetNames(typeof(SearchType)))
            {
                combobox.AppendText(item);
            }
            combobox.Active = 0;
            hbox.PackStart(combobox, false, false, 0);
            combobox.ShowAll();
            combobox.Changed += new EventHandler(OnChanged);
        }

        public void UnregisterUpdateDelegate()
        {
            // Nothing
        }

        void SearchResultsCb(object result)
        {
            // Just add all matching songs to the ListStore
            foreach (MusicInfo info in (result as MusicCollection).Songs)
            {
                tree.AppendValues(info.Filename);
            }
        }

        void EnqueueTreeSelection(TreeSelection ts)
        {
            TreeIter iter;
            // Return if nothing is selected
            if (!ts.TreeView.Model.GetIterFirst(out iter))
                return;

            TreeModel model;
            TreePath[] paths = ts.GetSelectedRows(out model);

            try
            {
                foreach (TreePath path in paths)
                {
                    model.GetIter(out iter, path);
                    string val = (string) model.GetValue(iter, 0);
                    if (conn.client != null)
                    {
                        conn.client.Add(val);
                    }
                }
                while (conn.client.ReadResponse()) {}
            }
            catch (MPD.MPDException)
            {}
        }

        void ShowMenu()
        {
            bool selected = view.Selection.CountSelectedRows() > 0;
            gxml_menu.GetWidget("play").Sensitive = selected;
            gxml_menu.GetWidget("add").Sensitive = selected;

            (gxml_menu.GetWidget("find_popup") as Menu).Popup();
        }

        /*
         * Main gui callbacks
         */
        void OnChanged(object o, EventArgs args)
        {
            if (MpcSharp.Settings.IncrementalSearch)
                entry.Activate();
        }

        void OnActivate(object o, EventArgs args)
        {
            // Clear old results
            tree.Clear();

            string what = entry.Text;

            // Require at least two characters
            if (what.Length < 2)
                return;
            try
            {
                TreeIter iter;
                combobox.GetActiveIter(out iter);
                string text = (string) combobox.Model.GetValue(iter, 0);
                
                SearchType type = (SearchType) SearchType.Parse(
                        typeof(SearchType), text, true);
                // Send command to mpd
                if (what != String.Empty && conn.client != null)
                {
                    conn.client.Search(type, what, false, entryCb);
                    while (conn.client.ReadResponse())
                    {}
                }
            }
            catch (ArgumentException e)
            {
                Console.WriteLine(e);
            }
            catch (MPD.MPDException e)
            {
                Console.WriteLine(e);
                conn.client.CleanCache();
            }
        }

        void OnRowActivated(object o, RowActivatedArgs args){
            EnqueueTreeSelection(view.Selection);
        }

        [GLib.ConnectBefore]
        void OnButtonPress(object o, ButtonPressEventArgs args){
            if (args.Event.Button == 3)
                args.RetVal = (o as TreeView).Selection.CountSelectedRows() > 1;
        }

        void OnButtonRelease(object o, ButtonReleaseEventArgs args){
            if (args.Event.Button == 3)
                ShowMenu();
            args.RetVal = true;
        }
        void OnPopupMenu(object o, PopupMenuArgs args){
            ShowMenu();
        }

        /*
         * Menu callbacks
         */
        void OnMenuPlay(object o, EventArgs args)
        {
            try
            {
                if (conn.client != null)
                {
                    conn.client.Clear();
                    EnqueueTreeSelection(view.Selection);
                    conn.client.Play();
                    conn.client.ReadResponse();
                }
            }
            catch (MPD.MPDException e)
            {
                Console.WriteLine(e);
            }
        }

        void OnMenuAdd(object o, EventArgs args)
        {
            EnqueueTreeSelection(view.Selection);
            try
            {
                if (conn.client != null)
                {
                    conn.client.Play();
                    conn.client.ReadResponse();
                }
            }
            catch (MPD.MPDException e)
            {
                Console.WriteLine(e);
            }
        }
    }
}
